      {******************************************************************}
      { PaletteForm                                                      }
      {                                                                  }
      { home page : http://www.winningcubed.de                           }
      { email     : martin.walter@winningcubed.de                        }
      {                                                                  }
      { date      : 20-09-2005                                           }
      {                                                                  }
      { Use of this file is permitted for commercial and non-commercial  }
      { use, as long as the author is credited.                          }
      { This file (c) 2005 Martin Walter                                 }
      {                                                                  }
      { This Software is distributed on an "AS IS" basis, WITHOUT        }
      { WARRANTY OF ANY KIND, either express or implied.                 }
      {                                                                  }
      { *****************************************************************}

unit PaletteForm;

interface

uses
  Windows, Forms, Classes, Controls, Messages, Graphics;

const
  MINFORMWIDTH = 50;
  CM_CANCEL = WM_USER + 44;

type
  THeaderStyle = (hsToolWindow, hsRegular);

  TButtonColors = class(TPersistent)
  private
    FControl: TControl;
    FFrameColor: TColor;
    FFaceColor: TColor;
    FTextColor: TColor;
    FOnChange: TNotifyEvent;
    procedure SetColor(Index: Integer; Value: TColor);
  protected
    procedure Change; virtual;
    procedure AssignTo(Dest: TPersistent); override;
    property Control: TControl read FControl;
  public
    constructor Create(Control: TControl); virtual;
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  published
    property FrameColor: TColor index 0 read FFrameColor write SetColor;
    property FaceColor: TColor index 1 read FFaceColor write SetColor;
    property TextColor: TColor index 2 read FTextColor write SetColor;
  end;

  TCancelThread = class(TThread)
  private
    FControlHandle: THandle;
    procedure SetControlHandle(const Value: THandle);
  published
    procedure Execute; override;
    property ControlHandle: THandle read FControlHandle write SetControlHandle;
  end;

  TMWKPaletteForm = class(TCustomForm)
  private
    FOnMinimize: TNotifyEvent;
    FHeaderHeight: Integer;
    FFrameSize: Integer;
    FHeaderRect: TRect;
    FInnerRect: TRect;
    FOldBounds: TRect;
    FCloseRect: TRect;
    FMinimizeRect: TRect;
    FMousePos: TPoint;
    FStyle: THeaderStyle;
    FCursorPos: Integer;
    FCaptionFont: TLogFontW;
    FOldHeight: Integer;
    FMinimized: Boolean;
    FOverClose: Boolean;
    FOverMinimize: Boolean;
    FCloseDown: Boolean;
    FMinimizeDown: Boolean;
    FResizeable: Boolean;
    FShowMinimize: Boolean;
    FShowClose: Boolean;
    FLeftDown: Boolean;
    FDropShadow: Boolean;
    FButtonColors: TButtonColors;
    FHotButtonColors: TButtonColors;
    FCaptionColor: TColor;
    FPressedButtonColors: TButtonColors;
    FFrameColor: TColor;
    FCaptionBarColor: TColor;
    FMinClientHeight: Integer;
    FCancelThread: TCancelThread;

    FIsClientWidthStored: Boolean;
    FIsClientHeightStored: Boolean;
    function GetWindowDC: HDC;
    procedure ReleaseWindowDC(DC: HDC);
    procedure PaintClose(DC: HDC);
    procedure PaintMinimize(DC: HDC);
    procedure PaintHeader(DC: HDC);
    procedure PaintFrame(DC: HDC);
    procedure DoOnColorChange(Sender: TObject);
    procedure CMCancel(var Message: TMessage); message CM_CANCEL;
    procedure WMThemeChanged(var Message: TMessage); message WM_THEMECHANGED;
    procedure WMNCCalcSize(var Message: TWMNCCalcSize); message WM_NCCALCSIZE;
    procedure WMNCPaint(var Message: TMessage); message WM_NCPAINT;
    procedure WMNCHitTest(var Message: TWMNCHitTest); message WM_NCHITTEST;
    procedure WMNCLBUTTONDOWN(var Message: TWMNCLButtonDown); message WM_NCLBUTTONDOWN;
    procedure WMNCLBUTTONUP(var Message: TWMNCLButtonUp); message WM_NCLBUTTONUP;
    procedure WMCAPTURECHANGED(var Message: TMessage); message WM_CAPTURECHANGED;
    procedure CMMOUSELEAVE(var Message: TMessage); message CM_MOUSELEAVE;
    procedure WMMOUSELEAVE(var Message: TMessage); message WM_MOUSELEAVE;
    procedure CMNCMOUSELEAVE(var Message: TMessage); message WM_NCMOUSELEAVE;

    procedure GetSizes;
    procedure SetOnMinimize(const Value: TNotifyEvent);
    procedure SetStyle(const Value: THeaderStyle);
    procedure SetResizeable(const Value: Boolean);
    procedure SetShowMinimize(const Value: Boolean);
    procedure SetShowClose(const Value: Boolean);
    procedure SetDropShadow(const Value: Boolean);
    procedure SetButtonColors(const Value: TButtonColors);
    procedure SetHotButtonColors(const Value: TButtonColors);
    procedure SetCaptionColor(const Value: TColor);
    procedure SetPressedButtonColors(const Value: TButtonColors);
    procedure SetFrameColor(const Value: TColor);
    procedure SetCaptionBarColor(const Value: TColor);
    function GetOriginalHeight: Integer;
    procedure SetMinimized(const Value: Boolean);
    procedure SetMinClientHeight(const Value: Integer);
    procedure SetClientWidth(Value: Integer);
    procedure SetClientHeight(Value: Integer);
    function GetClientHeight: Integer;
    function GetClientWidth: Integer;
    function GetWidth: Integer;
    procedure SetWidth(const Value: Integer);
    function GetHeight: Integer;
    procedure SetHeight(const Value: Integer);
  protected
    procedure DoOnClose; dynamic;
    procedure DoOnMinimize; dynamic;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
  public
    constructor Create(AOwner: TComponent); override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    property Minimized: Boolean read FMinimized write SetMinimized;
  published
    property Visible;
    property Enabled;
    property Caption;
    property Color;

    property Action;
    property ActiveControl;
    property Align;
    property AlphaBlend default False;
    property AlphaBlendValue default 255;
    property Anchors;
    property AutoScroll;
    property BiDiMode;
    property TransparentColor default False;
    property TransparentColorValue default 0;
    property Constraints;
    property Ctl3D;
    property UseDockManager;
    property DefaultMonitor;
    property DockSite;
    property DragKind;
    property DragMode;
    property Font;
    property FormStyle;
    property Height: Integer read GetHeight write SetHeight;
    property HelpFile;
    property HorzScrollBar;
    property KeyPreview;
    property Menu;
    property OldCreateOrder;
    property ObjectMenuItem;
    property ParentBiDiMode;
    property PixelsPerInch;
    property PopupMenu;
    property Position;
    property PrintScale;
    property Scaled;
    property ScreenSnap default False;
    property ShowHint;
    property SnapBuffer default 10;
    property VertScrollBar;
    property Width: Integer read GetWidth write SetWidth;
    property WindowState;
    property WindowMenu;
    property OnActivate;
    property OnCanResize;
    property OnClick;
    property OnClose;
    property OnCloseQuery;
    property OnConstrainedResize;
    property OnContextPopup;
    property OnCreate;
    property OnDblClick;
    property OnDestroy;
    property OnDeactivate;
    property OnDockDrop;
    property OnDockOver;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnGetSiteInfo;
    property OnHide;
    property OnHelp;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnMouseWheel;
    property OnMouseWheelDown;
    property OnMouseWheelUp;
    property OnPaint;
    property OnResize;
    property OnShortCut;
    property OnShow;
    property OnStartDock;
    property OnUnDock;

    property OnMinimize: TNotifyEvent read FOnMinimize write SetOnMinimize;
    property Style: THeaderStyle read FStyle write SetStyle;
    property DropShadow: Boolean read FDropShadow write SetDropShadow;
    property Resizeable: Boolean read FResizeable write SetResizeable;
    property ShowMinimize: Boolean read FShowMinimize write SetShowMinimize;
    property ShowClose: Boolean read FShowClose write SetShowClose;
    property ButtonColors: TButtonColors read FButtonColors write SetButtonColors;
    property HotButtonColors: TButtonColors read FHotButtonColors write SetHotButtonColors;
    property PressedButtonColors: TButtonColors read FPressedButtonColors write SetPressedButtonColors;
    property CaptionColor: TColor read FCaptionColor write SetCaptionColor;
    property CaptionBarColor: TColor read FCaptionBarColor write SetCaptionBarColor;
    property FrameColor: TColor read FFrameColor write SetFrameColor;
    property OriginalHeight: Integer read GetOriginalHeight;
    property MinClientHeight: Integer read FMinClientHeight write SetMinClientHeight;
    property ClientWidth: Integer read GetClientWidth write SetClientWidth stored FIsClientWidthStored;
    property ClientHeight: Integer read GetClientHeight write SetClientHeight stored FIsClientHeightStored;
  end;


implementation

uses
  SysUtils, Math;

function IsInRect(Rect: TRect; X, Y: Integer): Boolean;
begin
  Result := (X >= Rect.Left) and (X < Rect.Right) and
    (Y >= Rect.Top) and (Y < Rect.Bottom);
end;

function ActivateDropShadow(const Handle: THandle): Boolean;

  function IsXP: Boolean;
  begin
    Result := (Win32Platform = VER_PLATFORM_WIN32_NT) and
      CheckWin32Version(5, 1);
  end;

const
  SPI_SETDROPSHADOW = $1025;
  CS_DROPSHADOW = $00020000;
begin
  if IsXP and SystemParametersInfo(SPI_SETDROPSHADOW, 0, Pointer(True), 0) then
  begin
    Result := SetClassLong(Handle, GCL_STYLE, GetClassLong(Handle, GCL_STYLE) or CS_DROPSHADOW) <> 0;
    if Result then
      SendMessage(Handle, CM_RECREATEWND, 0, 0);
  end else
    Result := False;
end;

{ TMWKPaletteForm }

constructor TMWKPaletteForm.Create(AOwner: TComponent);
var
  MouseEvent: tagTRACKMOUSEEVENT;
  DC: HDC;
begin
  FCursorPos := -1;
  FShowClose := True;
  FShowMinimize := True;
  FResizeable := True;
  FDropShadow := True;

  FButtonColors := TButtonColors.Create(Self);
  FButtonColors.FrameColor := clActiveCaption;
  FButtonColors.FaceColor := clActiveCaption;
  FButtonColors.TextColor := clBtnText;
  FButtonColors.OnChange := DoOnColorChange;

  FHotButtonColors := TButtonColors.Create(Self);
  FHotButtonColors.FrameColor := clActiveCaption;
  FHotButtonColors.FaceColor := RGB(194, 207, 229);
  FHotButtonColors.TextColor := clBtnText;
  FHotButtonColors.OnChange := DoOnColorChange;

  FPressedButtonColors := TButtonColors.Create(Self);
  FPressedButtonColors.FrameColor := clActiveCaption;
  FPressedButtonColors.FaceColor := RGB(153, 175, 212);
  FPressedButtonColors.TextColor := RGB(68, 68, 68);
  FPressedButtonColors.OnChange := DoOnColorChange;

  FCaptionColor := clCaptionText;
  FCaptionBarColor := clActiveCaption;
  FFrameColor := clActiveCaption;

  FIsClientWidthStored := False;
  FIsClientHeightStored := False;

  inherited;
  BorderStyle := bsNone;
  FormStyle := fsStayOnTop;
  GetSizes;
  DC := GetWindowDC;
  PaintFrame(DC);
  PaintHeader(DC);
  ReleaseWindowDC(DC);
  if FDropShadow and not (csDesigning in ComponentState) then
    ActivateDropShadow(Handle);

  with MouseEvent do begin
    cbSize := SizeOf(MouseEvent);
    dwFlags := TME_LEAVE or $10;
    hwndTrack := Handle;
    dwHoverTime := HOVER_DEFAULT;
  end;
  TrackMouseEvent(MouseEvent);
end;

function TMWKPaletteForm.GetWindowDC: HDC;
var
  DC: HDC;
  RC, RW: TRect;
begin
  DC := Windows.GetWindowDC(Handle);
  Windows.GetClientRect(Handle, RC);
  GetWindowRect(Handle, RW);
  MapWindowPoints(0, Handle, RW, 2);
  OffsetRect(RC, -RW.Left, -RW.Top);
  ExcludeClipRect(DC, RC.Left, RC.Top, RC.Right, RC.Bottom);
  Result := DC;
end;

procedure TMWKPaletteForm.ReleaseWindowDC(DC: HDC);
begin
  ReleaseDC(Handle, DC);
end;

procedure TMWKPaletteForm.PaintClose(DC: HDC);
var
  Pen, OldPen: HPen;
  Brush, OldBrush: HBrush;
  P: TPoint;
  FrameColor, FaceColor, TextColor: TColor;
begin
  if not FShowClose then
    Exit;
  if FOverClose or FCloseDown then
  begin
    if FOverClose and FCloseDown then
    begin
      FrameColor := FPressedButtonColors.FrameColor;
      FaceColor := FPressedButtonColors.FaceColor;
      TextColor := FPressedButtonColors.TextColor;
    end else
      begin
        FrameColor := FHotButtonColors.FrameColor;
        FaceColor := FHotButtonColors.FaceColor;
        TextColor := FHotButtonColors.TextColor;
      end;
  end else
    begin
      FrameColor := FButtonColors.FrameColor;
      FaceColor := FButtonColors.FaceColor;
      TextColor := FButtonColors.TextColor;
    end;

  Pen := CreatePen(BS_SOLID, 1, ColorToRGB(FrameColor));
  Brush := CreateSolidBrush(ColorToRGB(FaceColor));

  OldPen := SelectObject(DC, Pen);
  OldBrush := SelectObject(DC, Brush);

  Rectangle(DC, FCloseRect.Left, FCloseRect.Top,
    FCloseRect.Right, FCloseRect.Bottom);

  SelectObject(DC, OldPen);
  SelectObject(DC, OldBrush);

  DeleteObject(Pen);
  DeleteObject(Brush);

  Pen := CreatePen(BS_SOLID, 2, ColorToRGB(TextColor));
  OldPen := SelectObject(DC, Pen);

  MoveToEx(DC, FCloseRect.Left + 5, FCloseRect.Top + 5, @P);
  LineTo(DC, FCloseRect.Right - 5, FCloseRect.Bottom - 5);

  MoveToEx(DC, FCloseRect.Right - 5, FCloseRect.Top + 5, @P);
  LineTo(DC, FCloseRect.Left + 5, FCloseRect.Bottom - 5);

  SelectObject(DC, OldPen);
  DeleteObject(Pen);
end;

procedure TMWKPaletteForm.PaintMinimize(DC: HDC);
var
  Pen, OldPen: HPen;
  Brush, OldBrush: HBrush;
  P: TRect;
  FrameColor, FaceColor, TextColor: TColor;
begin
  if not FShowMinimize then
    Exit;
  if FOverMinimize or FMinimizeDown then
  begin
    if FOverMinimize and FMinimizeDown then
    begin
      FrameColor := FPressedButtonColors.FrameColor;
      FaceColor := FPressedButtonColors.FaceColor;
      TextColor := FPressedButtonColors.TextColor;
    end else
      begin
        FrameColor := FHotButtonColors.FrameColor;
        FaceColor := FHotButtonColors.FaceColor;
        TextColor := FHotButtonColors.TextColor;
      end;
  end else
    begin
      FrameColor := FButtonColors.FrameColor;
      FaceColor := FButtonColors.FaceColor;
      TextColor := FButtonColors.TextColor;
    end;

  Pen := CreatePen(BS_SOLID, 1, ColorToRGB(FrameColor));
  Brush := CreateSolidBrush(ColorToRGB(FaceColor));

  OldPen := SelectObject(DC, Pen);
  OldBrush := SelectObject(DC, Brush);

  Rectangle(DC, FMinimizeRect.Left, FminimizeRect.Top,
    FMinimizeRect.Right, FMinimizeRect.Bottom);

  SelectObject(DC, OldPen);
  SelectObject(DC, OldBrush);

  DeleteObject(Pen);
  DeleteObject(Brush);

  Pen := CreatePen(BS_SOLID, 1, ColorToRGB(TextColor));
  OldPen := SelectObject(DC, Pen);

  if FMinimized then
  begin
    Brush := CreateSolidBrush(ColorToRGB(TextColor));

    FrameRect(DC, Rect(FMinimizeRect.Left + 4, FMinimizeRect.Top + 4,
      FMinimizeRect.Right - 4, FMinimizeRect.Bottom - 4), Brush);
    DeleteObject(Brush);

    MoveToEx(DC, FMinimizeRect.Left + 4, FMinimizeRect.Top + 5, @P);
    LineTo(DC, FMinimizeRect.Right - 4, FMinimizeRect.Top + 5);
  end else
    begin
      MoveToEx(DC, FMinimizeRect.Left + 5, FMinimizeRect.Bottom - 6, @P);
      LineTo(DC, FMinimizeRect.Right - 5, FMinimizeRect.Bottom - 6);
      MoveToEx(DC, FMinimizeRect.Left + 5, FMinimizeRect.Bottom - 5, @P);
      LineTo(DC, FMinimizeRect.Right - 5, FMinimizeRect.Bottom - 5);
    end;

  SelectObject(DC, OldPen);
  DeleteObject(Pen);
end;

procedure TMWKPaletteForm.PaintHeader(DC: HDC);
var
  Brush: HBrush;
  Pen, OldPen: HPen;
  Font, OldFont: HFont;
  X, Y: Integer;
  Size: TSize;
  R: TRect;
  P: TPoint;
begin
  Brush := CreateSolidBrush(ColorTORGB(FCaptionBarColor));
  FillRect(DC, FHeaderRect, Brush);
  DeleteObject(Brush);

  Pen := CreatePen(BS_SOLID, 1, ColorToRGB(Color));
  OldPen := SelectObject(DC, Pen);

  MoveToEx(DC, FHeaderRect.Left, FHeaderRect.Bottom, @P);
  LineTo(DC, FHeaderRect.Right, FHeaderRect.Bottom);
  SelectObject(DC, OldPen);
  DeleteObject(Pen);

  Font := CreateFontIndirectW(FCaptionFont);
  OldFont := SelectObject(DC, Font);

  SetTextColor(DC, ColorToRGB(FCaptionColor));
  SetBKMode(DC, TRANSPARENT);
  GetTextExtentPoint32(DC, PChar(Caption), Length(Caption), Size);

  X := FHeaderRect.Left + 3;
  Y := FHeaderRect.Top + (FHeaderHeight - Size.cy) div 2;
  R := FHeaderRect;
  if FShowMinimize then
    R.Right := FMinimizeRect.Left
  else
    if FShowClose then
      R.Right := FCloseRect.Left;

  ExtTextOut(DC, X, Y, ETO_CLIPPED, @R,
    PChar(Caption), Length(Caption), nil);

  SelectObject(DC, OldFont);
  DeleteObject(Font);

  PaintClose(DC);
  PaintMinimize(DC);
end;

procedure TMWKPaletteForm.PaintFrame(DC: HDC);
var
  Brush: HBrush;
  C: Integer;
  R: TRect;
begin
  Brush := CreateSolidBrush(ColorTORGB(FFrameColor));
  R := Rect(0, 0, Width, Height);

  for C := 0 to FFrameSize - 1 do
  begin
    FrameRect(DC, R, Brush);
    InflateRect(R, -1, -1);
  end;
  DeleteObject(Brush);
  Brush := CreateSolidBrush(ColorTORGB(Color));
  FrameRect(DC, R, Brush);
  DeleteObject(Brush);
end;

procedure TMWKPaletteForm.DoOnColorChange(Sender: TObject);
var
  DC: HDC;
begin
  DC := GetWindowDC;
  PaintFrame(DC);
  PaintHeader(DC);
  ReleaseWindowDC(DC);
end;

procedure TMWKPaletteForm.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  inherited;
  GetSizes;
end;

procedure TMWKPaletteForm.GetSizes;
var
  NC: TNonClientMetricsW;
begin
  if Style = hsRegular then
    FHeaderHeight := GetSystemMetrics(SM_CYCAPTION)
  else
    FHeaderHeight := GetSystemMetrics(SM_CYSMCAPTION);

  FFrameSize := GetSystemMetrics(SM_CXEDGE);

  FHeaderRect := Rect(FFrameSize + 1, FFrameSize + 1,
    Width - (FFrameSize + 1), FFrameSize + 1 + FHeaderHeight);
  FInnerRect := Rect(FFrameSize, FFrameSize + FHeaderHeight + 1,
    Width - FFrameSize, Height - FFrameSize);

  FillChar(NC, SizeOf(NC), 0);
  NC.cbSize := SizeOf(NC);
  SystemParametersInfoW(SPI_GETNONCLIENTMETRICS, SizeOf(NC), @NC, 0);

  if Style = hsRegular then
    FCaptionFont := NC.lfCaptionFont
  else
    FCaptionFont := NC.lfSmCaptionFont;

  FCloseRect := Rect(FHeaderRect.Right - FHeaderHeight, FHeaderRect.Top,
    FHeaderRect.Right, FHeaderRect.Bottom);

  if FShowClose then
    FMinimizeRect := Rect(FHeaderRect.Right - (2 * FHeaderHeight) - 1,
      FHeaderRect.Top, FHeaderRect.Right - FheaderHeight - 1, FHeaderRect.Bottom)
  else
    FMinimizeRect := Rect(FHeaderRect.Right - FHeaderHeight, FHeaderRect.Top,
      FHeaderRect.Right, FHeaderRect.Bottom);
end;

procedure TMWKPaletteForm.SetOnMinimize(const Value: TNotifyEvent);
begin
  FOnMinimize := Value;
end;

procedure TMWKPaletteForm.SetStyle(const Value: THeaderStyle);
begin
  if Value = FStyle then
    Exit;
  FStyle := Value;
  GetSizes;
  Invalidate;
end;

procedure TMWKPaletteForm.CMCancel(var Message: TMessage);
begin
  ReleaseCapture;
end;

procedure TMWKPaletteForm.WMThemeChanged(var Message: TMessage);
begin
  GetSizes;
end;

procedure TMWKPaletteForm.WMNCCalcSize(var Message: TWMNCCalcSize);
begin
  inherited;

  if not (csDesigning in ComponentState) then
    with Message.CalcSize_Params^ do
    begin
      InflateRect(rgrc[0], -FFrameSize - 1, -FFrameSize - 1);
      Inc(rgrc[0].Top, FHeaderHeight + 1);
      rgrc[0].Top := Min(rgrc[0].Top, rgrc[0].Bottom);
    end;
end;

procedure TMWKPaletteForm.WMNCPaint(var Message: TMessage);
var
  DC: HDC;
begin
  if not (csDesigning in ComponentState) then
  begin
    DC := GetWindowDC;
    try
      PaintFrame(DC);
      PaintHeader(DC);
    finally
      ReleaseWindowDC(DC);
    end;
  end;

  inherited;
end;

procedure TMWKPaletteForm.WMNCHitTest(var Message: TWMNCHitTest);
var
  P: TPoint;
  RC, RW, CL: TRect;
  X, Y: Integer;
  DC: HDC;
begin
  if csDesigning in ComponentState then
  begin
    FCursorPos := -1;
    Inherited;
    //Message.Result := HTCLIENT;
    Exit;
  end;

  Message.Result := 0;
  if FLeftDown and (FCursorPos in [0..7]) then
  begin
    Message.Result := HTBORDER;
    Exit;
  end;

  FCursorPos := -1;

  Windows.GetClientRect(Handle, RC);
  CL := RC;
  GetWindowRect(Handle, RW);
  OffsetRect(RC, -RW.Left, -RW.Top);

  Inc(CL.Left, FFrameSize + 1);
  Inc(CL.Top, FheaderHeight + FFrameSize + 1);
  Inc(CL.Right, FFrameSize + 1);
  Inc(CL.Bottom, FHeaderHeight + FFrameSize + 1);

  P := Point(Message.XPos + RC.Left, Message.YPos + RC.Top);
  X := P.X;
  Y := P.Y;

  if IsInRect(FHeaderRect, X, Y) then
  begin
    Message.Result := HTCAPTION;
    FCursorPos := 8;

    if FShowClose and IsInRect(FCloseRect, X, Y) then
      FCursorPos := 9;

    if FShowMinimize and IsInRect(FMinimizeRect, X, Y) then
      FCursorPos := 10;
  end else
    begin
      if IsInRect(CL, X, Y) then
        Message.Result := HTCLIENT;

      if (X >= 0) and (X < FFrameSize) then
      begin
        Message.Result := HTLEFT;
        FCursorPos := 3;

        if not FMinimized then
        begin
          if (Y >= 0) and (Y < FHeaderHeight) then
          begin
            Message.Result := HTTOPLEFT;
            FCursorPos := 0;
          end;

          if (Y >= Height - FHeaderHeight) and (Y < Height) then
          begin
            Message.Result := HTBOTTOMLEFT;
            FCursorPos := 5;
          end;
        end;
      end;

      if (X >= Width - FFrameSize) and (X < Width) then
      begin
        Message.Result := HTRIGHT;
        FCursorPos := 4;

        if not FMinimized then
        begin
          if (Y >= 0) and (Y < FHeaderHeight) then
          begin
            Message.Result := HTTOPRIGHT;
            FCursorPos := 2;
          end;

          if (Y >= Height - FHeaderHeight) and (Y < Height) then
          begin
            Message.Result := HTBOTTOMRIGHT;
            FCursorPos := 7;
          end;
        end;
      end;

      if (Y >= 0) and (Y < FFrameSize) and not FMinimized then
      begin
        Message.Result := HTTOP;
        FCursorPos := 1;

        if (X >= 0) and (X < FHeaderHeight) then
        begin
          Message.Result := HTTOPLEFT;
          FCursorPos := 0;
        end;

        if (X >= Width - FHeaderHeight) and (X < Width) then
        begin
          Message.Result := HTTOPRIGHT;
          FCursorPos := 2;
        end;
      end;

      if (Y >= Height - FFrameSize) and (Y < Height) and not FMinimized then
      begin
        Message.Result := HTBOTTOM;
        FCursorPos := 6;

        if (X >= 0) and (X < FHeaderHeight) then
        begin
          Message.Result := HTBOTTOMLEFT;
          FCursorPos := 5;
        end;

        if (X >= Width - FHeaderHeight) and (X < Width) then
        begin
          Message.Result := HTBOTTOMRIGHT;
          FCursorPos := 7;
        end;
      end;
    end;

  if (not FResizeable) and (FCursorPos in [0..7]) then
  begin
    FCursorPos := -1;
    Message.Result := HTBORDER;
  end;

  if not FLeftDown then
  begin
    if (not (FCursorPos in [9, 10])) and
       (FOverClose or FOverMinimize) then
    begin
      FOverClose := False;
      FOverMinimize := False;
      DC := GetWindowDC;
      PaintClose(DC);
      PaintMinimize(DC);
      ReleaseWindowDC(DC);
    end;
  end;

  if (FOverClose <> (FCursorPos = 9)) then
  begin
    FOverClose := not FOverClose;
    if FMinimizeDown then
      FOverClose := False;
    DC := GetWindowDC;
    PaintClose(DC);
    ReleaseWindowDC(DC);
  end;

  if (FOverMinimize <> (FCursorPos = 10)) then
  begin
    FOverMinimize := not FOverMinimize;
    if FCloseDown then
      FOverMinimize := False;
    DC := GetWindowDC;
    PaintMinimize(DC);
    ReleaseWindowDC(DC);
  end;

  if FCloseDown or FMinimizeDown then
    Message.Result := HTCAPTION;
end;

procedure TMWKPaletteForm.WMNCLBUTTONDOWN(var Message: TWMNCLButtonDown);
const
  SC_DRAGMOVE = $f012;

var
  RC, RW: TRect;
  P: TPoint;
  DC: HDC;
begin
  if csDesigning in ComponentState then
  begin
    inherited;
    Exit;
  end;

  BringToFront;
  Windows.GetClientRect(Handle, RC);
  GetWindowRect(Handle, RW);
  OffsetRect(RC, -RW.Left, -RW.Top);
  P := Point(Message.XCursor + RC.Left, Message.YCursor + RC.Top);

  FLeftDown := True;

  if FCursorPos in [0..7] then
  begin
    FCancelThread := TCancelThread.Create(True);
    FCancelThread.ControlHandle := Handle;
    FCancelThread.Resume;
    FOldBounds := BoundsRect;
    FMousePos := Point(Message.XCursor, Message.YCursor);
    SetCapture(Handle);
  end;

  if FCursorPos = 8 then
  begin
    ReleaseCapture;
    FLeftDown := False;
    SendMessage(Self.GetTopParentHandle, WM_SYSCOMMAND, SC_DRAGMOVE, 0);
    Exit;
  end;

  if FCursorPos = 9 then
  begin
    FCloseDown := True;

    DC := GetWindowDC;
    PaintHeader(DC);
    ReleaseWindowDC(DC);
    SetCapture(Handle);
  end;

  if FCursorPos = 10 then
  begin
    FMinimizeDown := True;

    DC := GetWindowDC;
    PaintHeader(DC);
    ReleaseWindowDC(DC);
    SetCapture(Handle);
  end;
end;

procedure TMWKPaletteForm.WMNCLBUTTONUP(var Message: TWMNCLButtonUp);
var
  DC: HDC;
begin
  if csDesigning in ComponentState then
  begin
    inherited;
    Exit;
  end;

  FLeftDown := False;

  if FCloseDown and FOverClose then
    DoOnClose;

  if FMinimizeDown and FOverMinimize then
    DoOnMinimize;

  FCloseDown := False;
  FMinimizeDown := False;

  if FCursorPos in [0..7, 9, 10] then
    ReleaseCapture;

  DC := GetWindowDC;
  PaintHeader(DC);
  ReleaseWindowDC(DC);
end;

procedure TMWKPaletteForm.WMCAPTURECHANGED(var Message: TMessage);
var
  DoPaint: Boolean;
  DC: HDC;
  Down: SmallInt;
begin
  if (csDesigning in ComponentState) then
  begin
    inherited;
    Exit;
  end;

  if Assigned(FCancelThread) then
    FCancelThread.Terminate;
  FreeAndNil(FCancelThread);

  if FLeftDown then
  begin
    if GetSystemMetrics(SM_SWAPBUTTON) > 0 then
      Down := GetAsyncKeyState(VK_RBUTTON)
    else
      Down := GetAsyncKeyState(VK_LBUTTON);

    if Down >= 0 then
      Exit;

    DoPaint := (FCloseDown and FOverClose) or
      (FMinimizeDown and FOverMinimize) or
      (FCursorPos in [0..7]);

    FLeftDown := False;
    FCloseDown := False;
    FMinimizeDown := False;

    if (FCursorPos in [0..7]) then
      SetBounds(FOldBounds.Left, FOldBounds.Top,
        FOldBounds.Right - FOldBounds.Left,
        FOldBounds.Bottom - FOldBounds.Top);


    if DoPaint then
    begin
      DC := GetWindowDC;
      PaintFrame(DC);
      PaintHeader(DC);
      ReleaseWindowDC(DC);
    end;
  end;
end;

procedure TMWKPaletteForm.CMMOUSELEAVE(var Message: TMessage);
var
  DC: HDC;
begin
  if (csDesigning in ComponentState) then
  begin
    inherited;
    Exit;
  end;

  if (FOverClose or FOverMinimize) then
  begin
    FOverClose := False;
    FOverMinimize := False;
    DC := GetWindowDC;
    PaintClose(DC);
    PaintMinimize(DC);
    ReleaseWindowDC(DC);
  end;
end;

procedure TMWKPaletteForm.WMMOUSELEAVE(var Message: TMessage);
var
  DC: HDC;
begin
  if (csDesigning in ComponentState) then
  begin
    inherited;
    Exit;
  end;

  if (FOverClose or FOverMinimize) then
  begin
    FOverClose := False;
    FOverMinimize := False;
    DC := GetWindowDC;
    PaintClose(DC);
    PaintMinimize(DC);
    ReleaseWindowDC(DC);
  end;
end;

procedure TMWKPaletteForm.CMNCMOUSELEAVE(var Message: TMessage);
var
  DC: HDC;
begin
  if (csDesigning in ComponentState) then
  begin
    inherited;
    Exit;
  end;

  if (FOverClose or FOverMinimize) then
  begin
    FOverClose := False;
    FOverMinimize := False;
    DC := GetWindowDC;
    PaintClose(DC);
    PaintMinimize(DC);
    ReleaseWindowDC(DC);
  end;
end;

procedure TMWKPaletteForm.MouseMove(Shift: TShiftState; X, Y: Integer);
var
  RW, R, BR: TRect;
  P: TPoint;
  DC: HDC;
begin
  if (csDesigning in ComponentState) then
  begin
    inherited;
    Exit;
  end;

  if FLeftDown and not (FCloseDown or FMinimizeDown) and
     (FCursorPos in [0..7]) then
  begin
    R := FOldBounds;
    P := Mouse.CursorPos;

    if FCursorPos in [0, 3, 5] then
    begin
      Inc(R.Left, P.X - FMousePos.X);
      if R.Right - R.Left < MINFORMWIDTH then
        R.Left := R.Right - MINFORMWIDTH;
    end;

    if FCursorPos in [0, 1, 2] then
    begin
      Inc(R.Top, P.Y - FMousePos.Y);
      if R.Bottom - R.Top < FHeaderHeight + 2 * FFrameSize + 2 then
        R.Top := R.Bottom - (FHeaderHeight + 2 * FFrameSize + 2);
    end;

    if FCursorPos in [2, 4, 7] then
    begin
      Inc(R.Right, P.X - FMousePos.X);
      if R.Right - R.Left < MINFORMWIDTH then
        R.Right := R.Left + MINFORMWIDTH;
    end;

    if FCursorPos in [5, 6, 7] then
    begin
      Inc(R.Bottom, P.Y - FMousePos.Y);
      if R.Bottom - R.Top < FHeaderHeight + 2 * FFrameSize + 2 then
        R.Bottom := R.Top +  FHeaderHeight + 2 * FFrameSize + 2;
    end;

    BR := BoundsRect;
    if (BR.Left <> R.Left) or (BR.Top <> R.Top) or
       (BR.Right <> R.Right) or (BR.Bottom <> R.Bottom) then
    begin
      SetBounds(R.Left, R.Top, R.Right - R.Left, R.Bottom - R.Top);
      SendMessage(Handle, WM_ERASEBKGND, 0, 0);
      DC := GetWindowDC;
      PaintFrame(DC);
      if (R.Right - R.Left <> BR.Right - BR.Left) then
        PaintHeader(DC);
      ReleaseWindowDC(DC);
    end;
  end;

  if FCloseDown or FMinimizeDown then
  begin
    GetWindowRect(Handle, RW);
    P.X := Mouse.CursorPos.X - RW.Left;
    P.Y := Mouse.CursorPos.Y - RW.Top;

    if FCloseDown and (FOverClose <> IsInRect(FCloseRect, P.X, P.Y)) then
    begin
      FOverClose := not FOverClose;
      DC := GetWindowDC;
      PaintClose(DC);
      ReleaseWindowDC(DC);
    end;

    if FMinimizeDown and (FOverMinimize <> IsInRect(FMinimizeRect, P.X, P.Y)) then
    begin
      FOverMinimize := not FOverMinimize;
      DC := GetWindowDC;
      PaintMinimize(DC);
      ReleaseWindowDC(DC);
    end;
  end;

  inherited;
end;

procedure TMWKPaletteForm.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
var
  DC: HDC;
begin
  if (csDesigning in ComponentState) then
  begin
    inherited;
    Exit;
  end;

  FLeftDown := False;

  if FCloseDown and FOverClose then
    DoOnClose;

  if FMinimizeDown and FOverMinimize then
    DoOnMinimize;

  FCloseDown := False;
  FMinimizeDown := False;

  DC := GetWindowDC;
  PaintHeader(DC);
  ReleaseWindowDC(DC);

  inherited;
end;

procedure TMWKPaletteForm.DoOnClose;
begin
  Close;
end;

procedure TMWKPaletteForm.DoOnMinimize;
var
  DC: HDC;
begin
  FMinimized := not FMinimized;

  if Assigned(FOnMinimize) then
    FOnMinimize(Self);

  if FMinimized then
  begin
    FOldHeight := Height;
    Height := FFrameSize * 2 + 2 + FHeaderHeight + FMinClientHeight;
  end else
    Height := FOldHeight;

  DC := GetWindowDC;
  PaintFrame(DC);
  ReleaseWindowDC(DC);
end;

procedure TMWKPaletteForm.SetResizeable(const Value: Boolean);
begin
  FResizeable := Value;
end;

procedure TMWKPaletteForm.SetShowMinimize(const Value: Boolean);
var
  DC: HDC;
begin
  if FShowMinimize <> Value then
  begin
    FShowMinimize := Value;
    GetSizes;
    DC := GetWindowDC;
    PaintHeader(DC);
    ReleaseWindowDC(DC);
  end;
end;

procedure TMWKPaletteForm.SetShowClose(const Value: Boolean);
var
  DC: HDC;
begin
  if FShowClose <> Value then
  begin
    FShowClose := Value;
    GetSizes;
    DC := GetWindowDC;
    PaintHeader(DC);
    ReleaseWindowDC(DC);
  end;
end;

procedure TMWKPaletteForm.SetDropShadow(const Value: Boolean);
begin
  if FDropShadow = Value then
    Exit;
  FDropShadow := Value;
  if FDropShadow and not (csDesigning in ComponentState) then
    ActivateDropShadow(Handle);
end;

procedure TMWKPaletteForm.SetButtonColors(const Value: TButtonColors);
begin
  FButtonColors.Assign(Value);
end;

procedure TMWKPaletteForm.SetHotButtonColors(const Value: TButtonColors);
begin
  FHotButtonColors.Assign(Value);
end;

procedure TMWKPaletteForm.SetCaptionColor(const Value: TColor);
begin
  FCaptionColor := Value;
  DoOnColorChange(nil);
end;

procedure TMWKPaletteForm.SetPressedButtonColors(const Value: TButtonColors);
begin
  FPressedButtonColors.Assign(Value);
end;

procedure TMWKPaletteForm.SetFrameColor(const Value: TColor);
begin
  FFrameColor := Value;
  DoOnColorChange(nil);
end;

procedure TMWKPaletteForm.SetCaptionBarColor(const Value: TColor);
begin
  FCaptionBarColor := Value;
  DoOnColorChange(nil);
end;

function TMWKPaletteForm.GetOriginalHeight: Integer;
begin
  if FMinimized then
    Result := FOldHeight
  else
    Result := Height;
end;

procedure TMWKPaletteForm.SetMinimized(const Value: Boolean);
var
  DC: HDC;
begin
  if Value = FMinimized then
    Exit;
  DoOnMinimize;
  DC := GetWindowDC;
  PaintMinimize(DC);
  ReleaseWindowDC(DC);
end;

procedure TMWKPaletteForm.SetMinClientHeight(const Value: Integer);
begin
  FMinClientHeight := Value;
end;

procedure TMWKPaletteForm.SetClientHeight(Value: Integer);
begin
  if FMinimized then
    Exit;
  if csDesigning in ComponentState then
    inherited ClientHeight := Value
  else
    Height := Value + ((FFrameSize + 1) * 2 + FHeaderHeight + 1);
  FIsClientHeightStored := True;
end;

procedure TMWKPaletteForm.SetClientWidth(Value: Integer);
begin
  if csDesigning in ComponentState then
    inherited ClientWidth := Value
  else
    Width := Value + ((FFrameSize + 1) * 2);

  FIsClientWidthStored := True;
end;

function TMWKPaletteForm.GetClientHeight: Integer;
begin
  if csDesigning in ComponentState then
    Result := inherited ClientHeight
  else
    Result := Height - ((FFrameSize + 1) * 2 + FHeaderHeight + 1);
end;

function TMWKPaletteForm.GetClientWidth: Integer;
begin
  if csDesigning in ComponentState then
    Result := inherited ClientWidth
  else
    Result := Width - ((FFrameSize + 1) * 2);
end;

function TMWKPaletteForm.GetWidth: Integer;
begin
  Result := inherited Width;
end;

procedure TMWKPaletteForm.SetWidth(const Value: Integer);
begin
  inherited Width := Value;
  FIsClientWidthStored := False;
end;

function TMWKPaletteForm.GetHeight: Integer;
begin
  Result := inherited Height;
end;

procedure TMWKPaletteForm.SetHeight(const Value: Integer);
begin
  inherited Height := Value;
  FIsClientHeightStored := False;
end;

{ TButtonColors }

procedure TButtonColors.AssignTo(Dest: TPersistent);
begin
  inherited;
  if (Dest is TButtonColors) then
  begin
    TButtonColors(Dest).FFrameColor := FrameColor;
    TButtonColors(Dest).FFaceColor := FaceColor;
    TButtonColors(Dest).FTextColor := TextColor;
    TButtonColors(Dest).Change;
  end;
end;

procedure TButtonColors.Change;
begin
  if Assigned(FOnChange) then FOnChange(Self);
end;

constructor TButtonColors.Create(Control: TControl);
begin
  inherited Create;
  FControl := Control;
end;

procedure TButtonColors.SetColor(Index: Integer; Value: TColor);
begin
  case Index of
    0:
      if Value <> FFrameColor then
      begin
        FFrameColor := Value;
        Change;
      end;
    1:
      if Value <> FFaceColor then
      begin
        FFaceColor := Value;
        Change;
      end;
    2:
      if Value <> FTextColor then
      begin
        FTextColor := Value;
        Change;
      end;
  end;
end;

{ TCancelThread }

procedure TCancelThread.Execute;
var
  IsSent: Boolean;
begin
  IsSent := False;
  while not Terminated do
    if (GetAsyncKeyState(VK_ESCAPE) <> 0) and
       (FControlHandle <> 0) and
       (not IsSent) then
    begin
      PostMessage(FControlHandle, CM_CANCEL, 0, 0);
      IsSent := True;
    end;
end;

procedure TCancelThread.SetControlHandle(const Value: THandle);
begin
  FControlHandle := Value;
end;


initialization
   RegisterClass(TMWKPaletteForm);
finalization
   UnRegisterClass(TMWKPaletteForm);
end.
