      {******************************************************************}
      { RegisterPaletteForm                                              }
      {                                                                  }
      { home page : http://www.winningcubed.de                           }
      { email     : martin.walter@winningcubed.de                        }
      {                                                                  }
      { date      : 20-09-2005                                           }
      {                                                                  }
      { Use of this file is permitted for commercial and non-commercial  }
      { use, as long as the author is credited.                          }
      { This file (c) 2005 Martin Walter                                 }
      {                                                                  }
      { This Software is distributed on an "AS IS" basis, WITHOUT        }
      { WARRANTY OF ANY KIND, either express or implied.                 }
      {                                                                  }
      { *****************************************************************}

unit RegisterPaletteForm;

interface

procedure Register;

implementation

uses
  Windows, Classes, SysUtils, DesignIntf, DesignEditors,
  ToolsApi, TypInfo, PaletteForm;

type
  TMWKFormCustomModule = class(TCustomModule)
  public
    class function DesignClass: TComponentClass; override;
  end;

  HICON = LongWord;

  TMWKNewFormWizard = class(TNotifierObject, IOTAWizard, IOTARepositoryWizard, IOTAFormWizard)
  protected
    function ThisFormName: AnsiString;
    function ThisFormClass: TComponentClass;
  public
    // IOTAWizard
    function GetIDString: AnsiString;
    function GetName: AnsiString;
    function GetState: TWizardState;
    procedure Execute;
    // IOTARepositoryWizard
    function GetAuthor: AnsiString;
    function GetComment: AnsiString;
    function GetPage: AnsiString;
    function GetGlyph: HICON;
  end;

  TMWKNewFormCreator = class(TInterfacedObject, IOTACreator, IOTAModuleCreator)
  private
    FAncestorName: AnsiString;
  public
    // IOTACreator
    function GetCreatorType: AnsiString;
    function GetExisting: Boolean;
    function GetFileSystem: AnsiString;
    function GetOwner: IOTAModule;
    function GetUnnamed: Boolean;
    // IOTAModuleCreator
    function GetAncestorName: AnsiString;
    function GetImplFileName: AnsiString;
    function GetIntfFileName: AnsiString;
    function GetFormName: AnsiString;
    function GetMainForm: Boolean;
    function GetShowForm: Boolean;
    function GetShowSource: Boolean;
    function NewFormFile(const FormIdent, AncestorIdent: AnsiString): IOTAFile;
    function NewImplSource(const ModuleIdent, FormIdent, AncestorIdent: AnsiString): IOTAFile;
    function NewIntfSource(const ModuleIdent, FormIdent, AncestorIdent: AnsiString): IOTAFile;
    procedure FormCreated(const FormEditor: IOTAFormEditor);
  public
    constructor Create(const FormName, AncestorName: AnsiString);
  end;

procedure Register;
begin
  RegisterCustomModule(TMWKPaletteForm, TMWKFormCustomModule);
  RegisterPackageWizard(TMWKNewFormWizard.Create);
end;


class function TMWKFormCustomModule.DesignClass: TComponentClass;
begin
  Result := TMWKPaletteForm;
end;

procedure AddUnitToUses(Module: IOTAModule; UnitName: AnsiString);
const
  UnitFileSize = 8192;
var
  Editor: IOTASourceEditor;
  Reader: IOTAEditReader;
  Writer: IOTAEditWriter;
  Buffer, P: PAnsiChar;
  StartPos: Integer;
begin
  Buffer := StrAlloc(UnitFileSize);
  Editor := (Module.GetModuleFileEditor(0)) as IOTASourceEditor;
  try
    Reader := Editor.CreateReader;
    try
      StartPos := Reader.GetText(0, Buffer, UnitFileSize);
      P := StrPos(Buffer, 'uses'); // Locate uses
      P := StrPos(P, ';'); // Locate the semi-colon afterwards
      if Assigned(P) then
        StartPos := Integer(P - Buffer)
      else
        StartPos := -1;
    finally
      Reader := nil; { get rid of reader before we use writer }
    end;
    if StartPos <> -1 then
    begin
      Writer := Editor.CreateWriter;
      try
        Writer.CopyTo(StartPos);
        Writer.Insert(PAnsiChar(AnsiString(', ' + UnitName)));
      finally
        Writer := nil;
      end;
    end;
  finally
    Editor := nil;
    StrDispose(Buffer);
  end;
end;

function GetFirstModuleSupporting(const IID: TGUID): IOTAModule;
var
  ModuleServices: IOTAModuleServices;
  i: integer;
begin
  Result := nil;
  if Assigned(BorlandIDEServices) then
  begin
    // look for the first project
    ModuleServices := BorlandIDEServices as IOTAModuleServices;
    for i := 0 to ModuleServices.ModuleCount - 1 do
      if Supports(ModuleServices.Modules[i], IID, Result) then
        Break;
  end;
end;

function MyGetActiveProject: IOTAProject;
begin
  Result := ToolsAPI.GetActiveProject;
  if (Result = nil) then
    Result := GetFirstModuleSupporting(IOTAProject) as IOTAProject;
end;

{ TMWKNewFormWizard }

procedure TMWKNewFormWizard.Execute;
var
  Module: IOTAModule;
begin
  Module := (BorlandIDEServices as IOTAModuleServices).CreateModule(TMWKNewFormCreator.Create('', ThisFormName));

  AddUnitToUses(Module, GetTypeData(PTypeInfo(ThisFormClass.ClassInfo)).UnitName);
end;

function TMWKNewFormWizard.GetAuthor: AnsiString;
begin
  Result := 'Martin Walter';
end;

function TMWKNewFormWizard.GetComment: AnsiString;
begin
  Result := 'Erstellt ein Palettenfenster';
end;

function TMWKNewFormWizard.GetGlyph: HICON;
begin
  Result := 0;
end;

function TMWKNewFormWizard.GetIDString: AnsiString;
begin
    Result := 'MWK.Create_' + ThisFormName + '.Wizard';
end;

function TMWKNewFormWizard.GetName: AnsiString;
begin
  Result := ThisFormName;
end;

function TMWKNewFormWizard.GetPage: AnsiString;
begin
  Result := 'Neu';
end;

function TMWKNewFormWizard.GetState: TWizardState;
begin
  Result := [wsEnabled];
end;

function TMWKNewFormWizard.ThisFormClass: TComponentClass;
begin
  Result := TMWKPaletteForm;
end;

function TMWKNewFormWizard.ThisFormName: AnsiString;
begin
  Result := ThisFormClass.ClassName;
  Delete(Result, 1, 1); // drop the 'T'
end;

{ TMWKNewFormCreator }

constructor TMWKNewFormCreator.Create(const FormName,
  AncestorName: AnsiString);
begin
  inherited Create;
  FAncestorName := AncestorName;
end;

procedure TMWKNewFormCreator.FormCreated(const FormEditor: IOTAFormEditor);
begin
end;

function TMWKNewFormCreator.GetAncestorName: AnsiString;
begin
  Result := FAncestorName;
end;

function TMWKNewFormCreator.GetCreatorType: AnsiString;
begin
  Result := sForm;
end;

function TMWKNewFormCreator.GetExisting: Boolean;
begin
  Result := False;
end;

function TMWKNewFormCreator.GetFileSystem: AnsiString;
begin
  Result := '';
end;

function TMWKNewFormCreator.GetFormName: AnsiString;
begin
  Result := '';
end;

function TMWKNewFormCreator.GetImplFileName: AnsiString;
begin
  Result := '';
end;

function TMWKNewFormCreator.GetIntfFileName: AnsiString;
begin
  Result := '';
end;

function TMWKNewFormCreator.GetMainForm: Boolean;
begin
  Result := False;
end;

function TMWKNewFormCreator.GetOwner: IOTAModule;
begin
  Result := MyGetActiveProject;
end;

function TMWKNewFormCreator.GetShowForm: Boolean;
begin
  Result := True;
end;

function TMWKNewFormCreator.GetShowSource: Boolean;
begin
  Result := True;
end;

function TMWKNewFormCreator.GetUnnamed: Boolean;
begin
  Result := True;
end;

function TMWKNewFormCreator.NewFormFile(const FormIdent,
  AncestorIdent: AnsiString): IOTAFile;
begin
  Result := nil;
end;

function TMWKNewFormCreator.NewImplSource(const ModuleIdent, FormIdent,
  AncestorIdent: AnsiString): IOTAFile;
begin
  Result := nil;
end;

function TMWKNewFormCreator.NewIntfSource(const ModuleIdent, FormIdent,
  AncestorIdent: AnsiString): IOTAFile;
begin
  Result := nil;
end;


end.
